/*
 * Copyright (C) 2021 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.model;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import org.hibernate.Session;
import storybook.model.hbn.dao.SceneDAOImpl;
import storybook.model.hbn.entity.Chapter;
import storybook.model.hbn.entity.Scene;
import storybook.toolkit.DateUtil;
import storybook.toolkit.ListUtil;
import storybook.toolkit.comparator.ObjectComparator;
import storybook.toolkit.SbDuration;
import storybook.ui.MainFrame;
import storybook.ui.dialog.ExceptionDlg;

/**
 *
 * @author favdb
 */
public class SceneUtil {

	private static final String TT = "SceneUtil";

	private SceneUtil() {
		// empty
	}

	/**
	 * get list of Scenes which have a fixed date
	 *
	 * @param mainFrame
	 * @return
	 */
	public static List<Scene> getWithDates(MainFrame mainFrame) {
		//App.trace("SceneUtil.getWithDates(mainFrame)");
		if (mainFrame == null) {
			return new ArrayList<>();
		}
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		SceneDAOImpl sceneDao = new SceneDAOImpl(session);
		List<Scene> scenes = sceneDao.findAll();
		model.commit();
		if (scenes.isEmpty()) {
			return scenes;
		}
		if (!scenes.get(0).hasScenets()) {
			scenes.get(0).setScenets(new Timestamp(new Date().getTime()));
		}
		int i = 0;
		for (Scene scene : scenes) {
			if (scene.hasRelativescene()) {
				Date date = null;
				for (int j = i; j >= 0; j--) {
					Scene s = scenes.get(j);
					if (s.getId().equals(scene.getRelativesceneid())) {
						date = s.getDate();
						if (date != null) {
							if (scene.getRelativetime().isEmpty()
									|| SbDuration.isZero(scene.getRelativetime())) {
								date = DateUtil.add(date, s.getSbDuration());
							} else {
								date = DateUtil.add(s.getDate(), scene.getRelativetime());
							}
							break;
						}
					}
				}
				scene.setDate(date);
			} else if (!scene.hasScenets() && i > 0) {
				Scene s = scenes.get(i - 1);
				if (s.hasDate()) {
					scene.setDate(DateUtil.add(s.getDate(), s.getSbDuration()));
				}
			}
			i++;
		}
		return scenes;
	}

	/**
	 * find all distinct date scene
	 *
	 * @param scenes
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static List<Date> findDistinctDates(List<Scene> scenes) {
		//App.trace(TT + ".findDistinctDates(part " + App.traceEntity(part) + ")");
		List<Date> dates = new ArrayList<>();
		if (scenes == null || scenes.isEmpty()) {
			return dates;
		}
		for (Scene scene : scenes) {
			if (scene.hasScenets()) {
				dates.add(scene.getDate());
			}
		}
		dates = ListUtil.removeNullAndDuplicates(dates);
		Collections.sort(dates, new ObjectComparator());
		return dates;
	}

	/**
	 * renumber all scenes
	 *
	 * @param mainFrame
	 */
	public static void renumber(MainFrame mainFrame) {
		//App.trace(TT + ".renumber(mainFrame)");
		if (mainFrame.getPref().sceneIsRenumAuto()) {
			renumberAuto(mainFrame, null);
		} else {
			renumberInc(mainFrame, null, 1, mainFrame.getPref().sceneGetRenumInc());
		}
		List<Chapter> chapters = EntityUtil.findChapters(mainFrame, null);
		int start = 1;
		for (Chapter chapter : chapters) {
			if (mainFrame.getPref().sceneIsRenumAuto()) {
				renumberAuto(mainFrame, chapter);
			} else {
				start = renumberInc(mainFrame, chapter, start, mainFrame.getPref().sceneGetRenumInc());
				if (mainFrame.getPref().sceneIsRenumByChapter()) {
					start = 1;
				}
			}
		}
		mainFrame.setUpdated();
		mainFrame.cursorSetDefault();
	}

	/**
	 * renumber all scenes of a paricular chapter
	 *
	 * @param mainFrame
	 * @param chapter
	 */
	public static void renumber(MainFrame mainFrame, Chapter chapter) {
		//App.trace(TT + ".renumber(mainFrame, chapter=" + App.traceEntity(chapter) + ")");
		try {
			if (mainFrame.getPref().sceneIsRenumAuto()) {
				renumberAuto(mainFrame, chapter);
			} else {
				renumberInc(mainFrame, chapter, 1, mainFrame.getPref().sceneGetRenumInc());
			}
			mainFrame.setUpdated();
		} catch (Exception e) {
			ExceptionDlg.show("", e);
		}
	}

	/**
	 * automatic renumber scenes of a chapter
	 *
	 * @param mainFrame
	 * @param chapter
	 */
	public static void renumberAuto(MainFrame mainFrame, Chapter chapter) {
		//App.trace(TT + ".renumberAuto(mainFrame, chapter=" + App.traceEntity(chapter) + ")");
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		SceneDAOImpl dao = new SceneDAOImpl(session);
		List<Scene> scenes = dao.findByChapter(chapter);
		int partNumber = 0;
		int chapterNumber = 0;
		if (chapter != null) {
			partNumber = (chapter.hasPart() ? chapter.getPart().getNumber() * 100000 : 0);
			chapterNumber = chapter.getChapterno() * 1000;
		}
		if (chapterNumber > 99000) {
			chapterNumber = 99000;
		}
		int sceneNumber = 1;
		int newNum;
		int inc = 1;
		for (Scene scene : scenes) {
			newNum = partNumber + chapterNumber + (sceneNumber * 10);
			scene.setSceneno(newNum);
			session.update(scene);
			sceneNumber += inc;
		}
		model.commit();
		model.fireAgainScenes();
		mainFrame.setUpdated();
	}

	/**
	 * incrmental renumber scenes of a chapter
	 *
	 * @param mainFrame
	 * @param chapter
	 * @param start: starting number
	 * @param inc: increment
	 * @return last number used
	 */
	public static int renumberInc(MainFrame mainFrame, Chapter chapter, int start, int inc) {
		//App.trace(TT + ".renumberInc(mainFrame, chapter=" + App.traceEntity(chapter) + ", start=" + start + ", inc=" + inc + ")");
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		SceneDAOImpl dao = new SceneDAOImpl(session);
		List<Scene> scenes = dao.findByChapter(chapter);
		int sceneNumber = start;
		for (Scene scene : scenes) {
			scene.setSceneno(sceneNumber);
			session.update(scene);
			sceneNumber += inc;
		}
		model.commit();
		model.fireAgainScenes();
		mainFrame.setUpdated();
		return sceneNumber;
	}

	/**
	 * get the next number for a scene in the given chapter
	 *
	 * @param mainFrame
	 * @param chapter
	 * @return
	 */
	public static int getNextNumber(MainFrame mainFrame, Chapter chapter) {
		//App.trace(TT + ".getNextNumber(mainFrame, chapter=" + App.traceEntity(chapter) + ")");
		int number = EntityUtil.findScenes(mainFrame, chapter).size();
		if (mainFrame.getPref().sceneIsRenumAuto()) {
			if (chapter != null) {
				String snum = String.format("%02d%02d%02d0",
						(chapter.hasPart() ? chapter.getPart().getNumber() : 0),
						chapter.getChapterno(),
						number + 1);
				return Integer.parseInt(snum);
			}
			return (number + 1) * 10;
		} else {
			if (!mainFrame.getPref().sceneIsRenumByChapter()) {
				number = EntityUtil.findScenes(mainFrame).size();
			}
			return number + mainFrame.getPref().sceneGetRenumInc();
		}
	}

}
