/*
Storybook: Open Source software for novelists and authors.
Copyright (C) 2008 - 2012 Martin Mustun

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.model.book;

import i18n.I18N;
import java.io.File;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import org.hibernate.Session;
import storybook.App;
import storybook.Pref;
import storybook.model.H2File;
import storybook.model.Model;
import storybook.model.hbn.dao.*;
import storybook.model.hbn.entity.AbstractEntity;
import storybook.model.hbn.entity.Attribute;
import storybook.model.hbn.entity.Category;
import storybook.model.hbn.entity.Chapter;
import storybook.model.hbn.entity.Endnote;
import storybook.model.hbn.entity.Episode;
import storybook.model.hbn.entity.Event;
import storybook.model.hbn.entity.Gender;
import storybook.model.hbn.entity.Idea;
import storybook.model.hbn.entity.Item;
import storybook.model.hbn.entity.Itemlink;
import storybook.model.hbn.entity.Location;
import storybook.model.hbn.entity.Memo;
import storybook.model.hbn.entity.Part;
import storybook.model.hbn.entity.Person;
import storybook.model.hbn.entity.Plot;
import storybook.model.hbn.entity.Relationship;
import storybook.model.hbn.entity.Scene;
import storybook.model.hbn.entity.Strand;
import storybook.model.hbn.entity.Tag;
import storybook.model.hbn.entity.Taglink;
import storybook.toolkit.Markdown;
import storybook.toolkit.file.EnvUtil;
import storybook.toolkit.file.FileFilter;
import storybook.ui.MainFrame;

/**
 * @author martin
 *
 */
public class BookUtil {

	private BookUtil() {
		// nothing
	}

	public static Map<Object, Integer> getBookSize(MainFrame mainFrame) {
		Map<Object, Integer> sizes = new HashMap<>();
		// Get scenes
		sizes.clear();
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		PartDAOImpl partdao = new PartDAOImpl(session);
		List<Part> parts = partdao.findAllRoots();
		session.close();

		for (Part part : parts) {
			appendPartSizes(mainFrame, part, sizes);
		}

		return sizes;
	}

	private static int appendPartSizes(MainFrame mainFrame, Part part, Map<Object, Integer> sizes) {
		int ret = 0;
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		PartDAOImpl partdao = new PartDAOImpl(session);
		List<Part> subparts = partdao.getParts(part);
		List<Chapter> chapters = partdao.findChapters(part);
		session.close();
		for (Part subpart : subparts) {
			ret += appendPartSizes(mainFrame, subpart, sizes);
		}
		for (Chapter chapter : chapters) {
			int chapterSize = appendChapterSizes(mainFrame, chapter, sizes);
			ret += chapterSize;
		}
		sizes.put(part, ret);
		return ret;
	}

	private static int appendChapterSizes(MainFrame mainFrame, Chapter chapter, Map<Object, Integer> sizes) {
		int ret = 0;
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		SceneDAOImpl dao = new SceneDAOImpl(session);
		List<Scene> scenes = dao.findByChapter(chapter);
		session.close();
		for (Scene scene : scenes) {
			int sceneSize = scene.getWords();
			sizes.put(scene, sceneSize);
			ret += sceneSize;
		}
		sizes.put(chapter, ret);
		return ret;
	}

	public static H2File openDocumentDialog() {
		final JFileChooser fc = new JFileChooser();
		fc.setCurrentDirectory(new File(App.getInstance().preferences.getString(Pref.KEY.LASTOPEN_DIR, EnvUtil.getHomeDir().getAbsolutePath())));
		FileFilter filter = new FileFilter("db");
		fc.addChoosableFileFilter(filter);
		fc.setFileFilter(filter);
		int ret = fc.showOpenDialog(null);
		if (ret == JFileChooser.APPROVE_OPTION) {
			File file = fc.getSelectedFile();
			if (!file.exists()) {
				JOptionPane.showMessageDialog(null,
						I18N.getMsg("project.not.exist.text", file),
						I18N.getMsg("project.not.exist.title"),
						JOptionPane.ERROR_MESSAGE);
				return null;
			}
			H2File dbFile = new H2File(file);
			return dbFile;
		}
		return null;
	}

	public static AbstractEntity getNewEntity(Book.TYPE objtype) {
		//App.trace("BookUtil.getNewEntity(type="+objtype.name()+")");
		AbstractEntity entity = null;
		switch (objtype) {
			case ATTRIBUTE:
			case ATTRIBUTES:
				entity = (new Attribute());
				break;
			case CATEGORY:
			case CATEGORIES:
				entity = (new Category());
				break;
			case CHAPTER:
			case CHAPTERS:
				entity = (new Chapter());
				break;
			case ENDNOTE:
			case ENDNOTES:
				entity = (new Endnote());
				break;
			case EPISODE:
			case EPISODES:
				entity = (new Episode());
				break;
			case EVENT:
			case EVENTS:
				entity = (new Event());
				break;
			case GENDER:
			case GENDERS:
				entity = (new Gender());
				break;
			case IDEA:
			case IDEAS:
				entity = (new Idea());
				break;
			case ITEM:
			case ITEMS:
				entity = (new Item());
				break;
			case ITEMLINK:
			case ITEMLINKS:
				entity = (new Itemlink());
				break;
			case LOCATION:
			case LOCATIONS:
				entity = (new Location());
				break;
			case MEMO:
			case MEMOS:
				entity = (new Memo());
				break;
			case PART:
			case PARTS:
				entity = (new Part());
				break;
			case PERSON:
			case PERSONS:
				entity = (new Person());
				break;
			case PLOT:
			case PLOTS:
				entity = (new Plot());
				break;
			case RELATION:
			case RELATIONS:
				entity = (new Relationship());
				break;
			case SCENE:
			case SCENES:
				entity = (new Scene());
				break;
			case STRAND:
			case STRANDS:
				entity = (new Strand());
				break;
			case TAG:
			case TAGS:
				entity = (new Tag());
				break;
			case TAGLINK:
			case TAGLINKS:
				entity = (new Taglink());
				break;
			default:
				return null;
		}
		return (entity);
	}

	public static Integer getChars(MainFrame mainFrame, Chapter chapter) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		SceneDAOImpl dao = new SceneDAOImpl(session);
		List<Scene> scenes = dao.findAll();
		session.close();
		int nb = 0;
		for (Scene scene : scenes) {
			boolean b = false;
			if (chapter == null) {
				b = true;
			} else if (scene.getChapter().equals(chapter)) {
				b = true;
			}
			if (b) {
				nb += scene.getChars();
			}
		}
		return nb;
	}

	public static Integer getChars(MainFrame mainFrame) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		SceneDAOImpl dao = new SceneDAOImpl(session);
		List<Scene> scenes = dao.findAll();
		session.close();
		int nb = 0;
		for (Scene scene : scenes) {
			nb += scene.getChars();
		}
		return nb;
	}

	public static Integer getWords(MainFrame mainFrame, Chapter chapter) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		PartDAOImpl partdao = new PartDAOImpl(session);
		partdao.findAllRoots();
		SceneDAOImpl dao = new SceneDAOImpl(session);
		List<Scene> scenes = dao.findAll();
		session.close();
		int nb = 0;
		for (Scene s : scenes) {
			if ((chapter == null && s.getChapter() == null)
					|| (s.getChapter() != null && s.getChapter().equals(chapter))) {
				nb += s.getWords();
			}
		}
		return nb;
	}

	public static Integer getWords(MainFrame mainFrame) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		SceneDAOImpl dao = new SceneDAOImpl(session);
		List<Scene> scenes = dao.findAll();
		session.close();
		int nb = 0;
		for (Scene scene : scenes) {
			nb += scene.getWords();
		}
		return nb;
	}

	public static void convertTo(MainFrame mainFrame, boolean b) {
		// when b is true convert to Markdown else convert to HTML
		Model model = mainFrame.getBookModel();
		Session session;
		session = model.beginTransaction();
		convertEntity(b, session, new AttributeDAOImpl(session));
		convertEntity(b, session, new CategoryDAOImpl(session));
		convertEntity(b, session, new ChapterDAOImpl(session));
		convertEntity(b, session, new EndnoteDAOImpl(session));
		convertEntity(b, session, new GenderDAOImpl(session));
		convertEntity(b, session, new IdeaDAOImpl(session));
		convertEntity(b, session, new ItemDAOImpl(session));
		convertEntity(b, session, new ItemlinkDAOImpl(session));
		convertEntity(b, session, new LocationDAOImpl(session));
		convertEntity(b, session, new MemoDAOImpl(session));
		convertEntity(b, session, new PartDAOImpl(session));
		convertEntity(b, session, new PersonDAOImpl(session));
		convertEntity(b, session, new PlotDAOImpl(session));
		convertEntity(b, session, new RelationDAOImpl(session));
		convertEntity(b, session, new SceneDAOImpl(session));
		convertEntity(b, session, new StrandDAOImpl(session));
		convertEntity(b, session, new TagDAOImpl(session));
		convertEntity(b, session, new TaglinkDAOImpl(session));
		convertEntity(b, session, new EventDAOImpl(session));
		session.close();
	}

	public static void convertEntity(boolean b, Session session, SbGenericDAOImpl dao) {
		dao.setSession(session);
		List entities = dao.findAll();
		for (Object obj : entities) {
			boolean c = false;
			AbstractEntity entity = (AbstractEntity) obj;
			if (b && entity.getNotes().contains("</p>")) {
				entity.setNotes(Markdown.toMarkdown(entity.getNotes()));
				c = true;
			}
			if (!b && !entity.getNotes().contains("</p>")) {
				entity.setNotes(Markdown.toHtml(entity.getNotes()));
				c = true;
			}
			if (b && entity.getDescription().contains("</p>")) {
				entity.setDescription(Markdown.toMarkdown(entity.getDescription()));
				c = true;
			}
			if (!b && !entity.getDescription().contains("</p>")) {
				entity.setDescription(Markdown.toHtml(entity.getDescription()));
				c = true;
			}
			if (entity instanceof Scene) {
				if (b && ((Scene) entity).getSummary().contains("</p>")) {
					((Scene) entity).setSummary(Markdown.toMarkdown(((Scene) entity).getSummary()));
					c = true;
				}
				if (!b && !((Scene) entity).getSummary().contains("</p>")) {
					((Scene) entity).setSummary(Markdown.toHtml(((Scene) entity).getSummary()));
					c = true;
				}
			}
			if (c) {
				session.update(entity);
			}
		}
		//session.close();
	}

}
