/*
Storybook: Open Source software for novelists and authors.
Copyright (C) 2008 - 2015 Martin Mustun, Pete Keller

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.model.hbn.entity;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import javax.swing.Icon;
import org.w3c.dom.Node;
import storybook.App;
import storybook.model.EntityUtil;
import storybook.model.book.Book;
import storybook.toolkit.DateUtil;
import static storybook.toolkit.DateUtil.clearTime;
import storybook.toolkit.ListUtil;
import storybook.toolkit.swing.ColorUtil;

/**
 * @hibernate.class table="PERSON"
 */
public class Person extends AbstractEntity {

	private Gender gender = null;
	private String firstname = "";
	private String lastname = "";
	private String abbreviation = "";
	private Date birthday = null;
	private Date dayofdeath = null;
	private String occupation = "";
	private Integer color = 0;
	private Category category = null;
	private List<Category> categories = new ArrayList<>();
	private List<Attribute> attributes = new ArrayList<>();

	public Person() {
		super(Book.TYPE.PERSON, "110");
	}

	@Override
	public String getName() {
		if (super.getName().isEmpty()) {
			return (getFirstname() + " " + getLastname()).trim();
		}
		return super.getName();
	}

	public Person(Gender gender, String firstname, String lastname,
			String abbreviation, Date birthday, Date dayofdeath,
			String occupation, String description, Integer color, String notes,
			Category category, List<Attribute> attributes) {
		this();
		this.gender = gender;
		this.firstname = firstname;
		this.lastname = lastname;
		this.abbreviation = abbreviation;
		this.birthday = birthday;
		this.dayofdeath = dayofdeath;
		this.occupation = occupation;
		this.color = color;
		setNotes(notes);
		this.category = category;
		this.attributes = attributes;
		setAssistant("");
	}

	public Gender getGender() {
		return this.gender;
	}

	public void setGender(Gender gender) {
		this.gender = gender;
	}

	public String getFirstname() {
		return this.firstname == null ? "" : this.firstname;
	}

	public void setFirstname(String firstname) {
		this.firstname = firstname;
		setName((getFirstname() + " " + getLastname()).trim());
	}

	public String getLastname() {
		return this.lastname == null ? "" : this.lastname;
	}

	public void setLastname(String lastname) {
		this.lastname = lastname;
		setName((getFirstname() + " " + getLastname()).trim());
	}

	public String getFullName() {
		return (getFirstname() + " " + getLastname()).trim();
	}

	public String getFullNameAbbr() {
		return getFullName() + " [" + getAbbreviation() + "]";
	}

	public String getAbbreviation() {
		return this.abbreviation == null ? "" : this.abbreviation;
	}

	public void setAbbreviation(String abbreviation) {
		this.abbreviation = abbreviation;
	}

	public Date getBirthday() {
		return this.birthday;
	}

	public String getBirthdayToString() {
		return (birthday == null ? "" : birthday.toString());
	}

	public void setBirthday(Date birthday) {
		this.birthday = birthday;
	}

	public Date getDayofdeath() {
		return this.dayofdeath;
	}

	public String getDayofdeathToString() {
		return (dayofdeath == null ? "" : birthday.toString());
	}

	public void setDayofdeath(Date dayofdeath) {
		this.dayofdeath = dayofdeath;
	}

	public String getOccupation() {
		return this.occupation;
	}

	public void setOccupation(String occupation) {
		this.occupation = occupation;
	}

	public Integer getColor() {
		return (this.color);
	}

	public Color getJColor() {
		return (color == 0 ? new Color(0) : new Color(color));
	}

	public String getHTMLColor() {
		return "<span style=\""
				+ "background-color:" + ColorUtil.getHTML(getJColor())
				+ ";"
				+ "color:" + ColorUtil.getHTML(getJColor())
				+ ";\">"
				+ "&#x25ae;&#x25ae;"
				+ "</span> "
				+ ColorUtil.getHTML(getJColor());
	}

	public void setColor(Integer color) {
		this.color = color;
	}

	public void setJColor(Color color) {
		if (color == null) {
			this.color = 0;
			return;
		}
		this.color = color.getRGB();
	}

	public Category getCategory() {
		return this.category;
	}

	public void setCategory(Category category) {
		this.category = category;
	}

	public List<Category> getCategories() {
		if (this.categories == null) {
			return (new ArrayList<>());
		}
		return this.categories;
	}

	public void setCategories(List<Category> categories) {
		this.categories = categories;
	}

	public void addCategories(Category c) {
		categories.add(c);
		ListUtil.setUnique(categories);
	}

	public void removeCategories(Category c) {
		categories.remove(c);
	}

	public List<Attribute> getAttributes() {
		if (attributes == null) {
			attributes = new ArrayList<>();
		}
		return attributes;
	}

	public void setAttributes(List<Attribute> attributes) {
		this.attributes = attributes;
	}

	@Override
	public String getAbbr() {
		if (abbreviation == null) {
			return ("??");
		}
		return abbreviation;
	}

	@Override
	public Icon getIcon() {
		if (gender != null) {
			return gender.getIcon();
		}
		return super.getIcon();
	}

	@Override
	public Icon getIcon(int sz) {
		if (gender != null) {
			return gender.getIcon(sz);
		}
		return super.getIcon(sz);
	}

	@Override
	public boolean hasDate() {
		return dayofdeath != null || birthday != null;
	}

	public Boolean isDead(Date now) {
		if (getDayofdeath() == null || now == null) {
			return false;
		}
		return (now.after(getDayofdeath()));
	}

	public int calculateAge(Date now) {
		if (birthday == null) {
			return -1;
		}

		Calendar dateOfBirth = new GregorianCalendar();
		dateOfBirth.setTime(birthday);
		dateOfBirth = clearTime(dateOfBirth);

		if (isDead(now)) {
			Calendar death = new GregorianCalendar();
			death.setTime(getDayofdeath());
			death = clearTime(death);

			int age = death.get(Calendar.YEAR) - dateOfBirth.get(Calendar.YEAR);

			Calendar dateOfBirth2 = new GregorianCalendar();
			dateOfBirth2.setTime(birthday);
			dateOfBirth2 = clearTime(dateOfBirth2);
			dateOfBirth2.add(Calendar.YEAR, age);

			if (death.before(dateOfBirth2)) {
				age--;
			}
			return age;
		}

		Calendar today = new GregorianCalendar();
		today.setTime(now);

		int age = today.get(Calendar.YEAR) - dateOfBirth.get(Calendar.YEAR);

		dateOfBirth.add(Calendar.YEAR, age);

		if (today.before(dateOfBirth)) {
			age--;
		}
		return age;
	}

	@Override
	@SuppressWarnings("unchecked")
	public String toDetail(boolean detailed) {
		StringBuilder b = new StringBuilder();
		b.append(super.toHtmlDetailHeader(detailed));
		if (detailed) {
			b.append(getInfo("firstname", getFirstname()));
			b.append(getInfo("lastname", getLastname()));
			b.append(getInfo("abbreviation", getAbbreviation()));
		}
		if (getGender() != null) {
			b.append(getInfo("gender", getClean(getGender().getName())));
		}
		if (detailed) {
			if (birthday != null) {
				b.append(getInfo("birthday", getClean(getBirthday())));
			}
			if (dayofdeath != null) {
				b.append(getInfo("death", getClean(getDayofdeath())));
			}
			if (!occupation.isEmpty()) {
				b.append(getInfo("occupation", getOccupation()));
			}
		}
		b.append(getInfo("color", getHTMLColor()));
		if (getCategory() != null) {
			b.append(getInfo("category", getClean(getCategory().getId()) + " (" + getCategory().toString() + ")"));
		}
		if (detailed) {
			if (categories != null && !getCategories().isEmpty()) {
				b.append(getInfo("categories", EntityUtil.getIds((List) getCategories())));
			}
			if (App.isDev()) {
				b.append(getInfo("attributes", EntityUtil.getIds((List) getAttributes())));
			}
		}
		b.append(super.toHtmlDetailFooter(detailed));
		return (b.toString());
	}

	@Override
	public String toString() {
		if (isTransient()) {
			return "";
		}
		return (getName() + (hasNotes() ? "*" : ""));
	}

	@Override
	public String toCsv(String quoteStart, String quoteEnd, String separator) {
		StringBuilder b = new StringBuilder();
		b.append(quoteStart).append(getClean(this)).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(getGender())).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(firstname)).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(lastname)).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(abbreviation)).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(birthday)).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(dayofdeath)).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(occupation)).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(getJColor())).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(getCategory())).append(quoteEnd).append(separator);
		b.append(quoteStart);
		for (Attribute attr : getAttributes()) {
			b.append(attr.getId().toString()).append("/");
		}
		b.append(quoteEnd).append(separator);
		b.append(quoteStart).append(getDescription()).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getNotes()).append(quoteEnd).append("\n");
		return (b.toString());
	}

	@Override
	public String toHtml() {
		return (toCsv("<td>", "</td>", "\n"));
	}

	@Override
	public String toText() {
		return (toCsv("", "", "\t"));
	}

	@Override
	@SuppressWarnings("unchecked")
	public String toXml() {
		StringBuilder b = new StringBuilder();
		b.append(toXmlTab(1)).append("<person \n");
		b.append(xmlCommon());
		b.append(xmlAttribute("name", getFirstname() + " " + getLastname()));
		if (getGender() != null) {
			b.append(xmlAttribute("gender", getClean(getGender().getId())));
		}
		b.append(xmlAttribute("firstname", getFirstname()));
		b.append(xmlAttribute("lastname", getLastname()));
		b.append(xmlAttribute("abbreviation", getAbbreviation()));
		b.append(xmlAttribute("birthday", getClean(getBirthday())));
		b.append(xmlAttribute("death", getClean(getDayofdeath())));
		b.append(xmlAttribute("occupation", getOccupation()));
		b.append(xmlAttribute("color", getClean(getColor())));
		b.append(xmlAttribute("category", getClean(getCategory().getId())));
		b.append(xmlAttribute("categories", EntityUtil.getIds((List) getCategories())));
		b.append(toXmlTab(2)).append(">\n");
		for (Attribute attr : getAttributes()) {
			b.append(toXmlMeta(2, "attribute", "[" + attr.getKey() + "]" + attr.getValue(), false));
		}
		b.append(toXmlEnd());
		return (b.toString());
	}

	// NOTE les attributs et des catégories multiples sont ignorées
	public static Person fromXml(Node node) {
		Person p = new Person();
		p.setId(getXmlLong(node, "id"));
		p.setFirstname(getXmlString(node, "firstname"));
		p.setLastname(getXmlString(node, "lastname"));
		p.setName(p.getFirstname() + " " + p.getLastname());
		p.setAbbreviation(getXmlString(node, "abbreviation"));
		p.setBirthday(DateUtil.stdStringToDate(getXmlString(node, "birthday")));
		p.setDayofdeath(DateUtil.stdStringToDate(getXmlString(node, "death")));
		p.setOccupation(getXmlString(node, "occupation"));
		if (!getXmlString(node, "color").isEmpty()) {
			p.setColor(getXmlInteger(node, "color"));
		}
		p.setDescription(getXmlText(node, "description"));
		p.setNotes(getXmlText(node, "notes"));
		p.setAssistant(getXmlText(node, "assistant"));
		return (p);
	}

	@Override
	@SuppressWarnings("EqualsWhichDoesntCheckParameterClass")
	public boolean equals(Object obj) {
		if (!super.equals(obj)) {
			return false;
		}
		Person test = (Person) obj;
		boolean ret = true;
		ret = ret && equalsStringNullValue(getAbbreviation(), test.getAbbreviation());
		ret = ret && equalsStringNullValue(getFirstname(), test.getFirstname());
		ret = ret && equalsStringNullValue(getLastname(), test.getLastname());
		ret = ret && equalsObjectNullValue(getGender(), test.getGender());
		ret = ret && equalsDateNullValue(getBirthday(), test.getBirthday());
		ret = ret && equalsDateNullValue(getDayofdeath(), test.getDayofdeath());
		ret = ret && equalsIntegerNullValue(getColor(), test.getColor());
		ret = ret && equalsObjectNullValue(getCategory(), test.getCategory());
		ret = ret && equalsStringNullValue(getNotes(), test.getNotes());
		ret = ret && equalsListNullValue(getAttributes(), test.getAttributes());
		return ret;
	}

	@Override
	public int hashCode() {
		return hashPlus(super.hashCode(),
				abbreviation,
				firstname,
				lastname,
				gender,
				birthday,
				dayofdeath,
				color,
				category,
				categories,
				occupation,
				attributes);
	}

	@Override
	public int compareTo(AbstractEntity en) {
		Person o = (Person) en;
		if (category == null && o == null) {
			return 0;
		}
		if (category != null && o.getCategory() == null) {
			return -1;
		}
		if (o.getCategory() != null && category == null) {
			return -1;
		}
		int cmp = category.getSort().compareTo(o.getCategory().getSort());
		if (cmp == 0) {
			return getFullName().compareTo(o.getFullName());
		}
		return cmp;
	}

	public static Person find(List<Person> list, String str) {
		for (Person elem : list) {
			if (elem.getName().equals(str)) {
				return (elem);
			}
		}
		return (null);
	}

	public static Person find(List<Person> list, Long id) {
		for (Person elem : list) {
			if (elem.id.equals(id)) {
				return (elem);
			}
		}
		return (null);
	}

	public static List<String> getDefColumns() {
		List<String> list = AbstractEntity.getDefColumns(Book.TYPE.PERSON);
		list.add("firstname, 256");
		list.add("lastname, 256");
		list.add("abbreviation, 256");
		list.add("occupation, 256");
		return (list);
	}

	public static List<String> getTable() {
		List<String> ls = new ArrayList<>();
		String tableName = "person";
		AbstractEntity.getTable(tableName, ls);
		ls.add(tableName + ",gender_id,Integer,0");
		ls.add(tableName + ",firstname,String,256");
		ls.add(tableName + ",lastname,String,256");
		ls.add(tableName + ",abbreviation,String,256");
		ls.add(tableName + ",birthday,Date,0");
		ls.add(tableName + ",dayofdeath,Date,0");
		ls.add(tableName + ",occupation,String,256");
		ls.add(tableName + ",color,Integer,0");
		ls.add(tableName + ",category_id,Integer,0");
		ls.add(tableName + ",categories,Table.Category,0");
		ls.add(tableName + ",attributes,Table.Attribute,0");
		return (ls);
	}

}
